/* ***************
// quicknote.js (tabwidth=2)
// Implementation file for QuickNote  <http://quicknote.mozdev.org/>
//
// Author: Jed Brown (quicknote@jedbrown.net)
// Additional code taken from Cdn's Chrome Edit extension.
// Contributors:
//   Nickolay Ponomarev (asqueella@mail.ru)
//   Leung WC
//   ...
//
// Version: 0.5.8+ dev
****************** */

var QN_stringBundle;
// quicknote's document object (inserted here, because simple
// 'document.something' doesn't work when a function is
// called from a global overlay (such as overlay.xul) )
var QN_Document=null;


var QN_charsLeft = Array(QN_notesCount+1); // ..until autosave (when qnautosave>0)

// Desc: QN_noteLength[X] = PadX.textLength
//       Used in autosave impl.
// Note: Do not rely on this prop. value (since it isn't always in sync with
// PadX.textLength).
var QN_noteLength = Array(QN_notesCount+1);

////////////////////////////////////////////////////////////////////////////////
// OnLoad
//
// Desc: Starts everyting (loading, Tab Labels, caught text, etc.)
// Parameters: Pad to copy the selection (via arguments[0])
// Returns: nothing
////////////////////////////////////////////////////////////////////////////////
function QuickNote_OnLoad() {
// TODO -p5: find out what is the default state for *.collapsed
// (asq) actually I thought it was true for 3rd & 4th; false for 1st and 2nd.
// I also suspect that each time OnLoad is triggered, the new instance of .xul/.js and therefore
// of all the objects, including textboxes is created, so some of these statements /are/ redundant
// I was too lazy, however, to check that out, so I ended up adding this comment ;)
// (jed) No worries, it's not really a priority, well figure it out sooner or later.


//-- set up XUL attributes depending on prefs
  QuickNote_setupDocument(document);
  if(QN_Preferences.qnfirststart) {
    //to remove:
    // xxx padfontsize is not set correctly here at first time
    var alertText = QN_stringBundle.getString("qn.welcome");
    alert(alertText);
    window.close();
    window.open('qnsettings.xul', 'QuickNote - Settings',
                'chrome,centerscreen,resizable');
//    return;
  }

  var i;
  if(QN_Preferences.QN_hideButtonBox == false) { //quicknote.hidebuttonbox
    for(i=1; i<=QN_notesCount; i++)
      window.document.getElementById("buttonbox"+i).hidden = false;
  }

  for(i=1; i<=QN_Preferences.numberoftabs; i++)
    document.getElementById("Tab"+i).collapsed = false;
  for(i=QN_Preferences.numberoftabs+1; i<=QN_notesCount; i++)
    document.getElementById("Tab"+i).collapsed = true;

//-- load notes
  for(i=1; i<=QN_notesCount; i++)
    QuickNote_loadNote(i);

  // now catch the text sent to QuickNote via context menu item
  // "Send .. to QuickNote" (if any [text])
  var textRecipent = null; // for now; see catchText comment
  try {
    textRecipent = QuickNote_catchText(window.arguments[0]);
  } catch(e) {
  }

  if(!textRecipent) {
    // Gets the last tab that was opened and sets
    // the tab to focus via selectedIndex
    document.getElementById("qntabs").selectedIndex = QN_Preferences.QNLastTabOpened-1;
    var thePadId = "Pad" + QN_Preferences.QNLastTabOpened;
    var thePad = document.getElementById(thePadId);
    thePad.focus();
  } // otherwise focus goes to the tab that caught the text
}

// this function must be called from global overlays (ie. those that do not
// overlay the QuickNote window, eg. overlay.xul) prior to any other qn.js
// functions.
function QuickNote_setupDocument(doc)
{
  QN_Preferences.readPrefs();
  QN_Document = doc;
  QN_stringBundle = doc.getElementById("strings");
}


////////////////////////////////////////////////////////////////////////////////
// openQuickNote()
// Desc: Opens QuickNote depending on set pref ('.openqn')
////////////////////////////////////////////////////////////////////////////////
function QuickNote_openQuickNote() {
  QN_Preferences.howtopenQN = null;
  try {
    QN_Preferences.howtopenQN = nsPreferences.getIntPref('quicknote.openqn', null);
  } catch (e) { // foobar!
  }

  if(QN_Preferences.howtopenQN == '0') {
    toggleSidebar('viewQuickNoteSidebar');
  } else if(QN_Preferences.howtopenQN == '2'){
    QuickNote_openQuickNoteInTab();
	} else
  // default:
  openQNFloat();
}

// Open QuickNote as a tab 
//### taken from Download Manager Tweak dmextension.mozdev.org
function  QuickNote_openQuickNoteInTab(){
  var numTabs = gBrowser.mTabContainer.childNodes.length;
  var index = 0;
  var oldTab = getBrowser().selectedTab;

  if (oldTab.getAttribute('QuickNote') == 'true') {
  if (getWebNavigation().currentURI.spec == "chrome://quicknote/content/quicknote.xhtml") {
      BrowserCloseTabOrWindow();
      return;
    }
    else oldTab.removeAttribute('QuickNote');
  }

  var tempTab = null;
  while (index < numTabs) {
    tempTab = gBrowser.mTabContainer.childNodes[index];

    if (tempTab.getAttribute('QuickNote') == 'true') {
      if (tempTab.label == 'QuickNote' || tempTab.label == 'chrome://quicknote/content/quicknote.xhtml'){ 
        getBrowser().selectedTab = tempTab;
        break;
      }else {
        getBrowser().selectedTab = tempTab;
        var url = getWebNavigation().currentURI.spec;
        if (url == 'chrome://quicknote/content/quicknote.xhtml') break;
        else tempTab.removeAttribute('QuickNote');
    }
  }
    else index = index+1;
  }

  // If no QuickNote tab is open, then create one
  if (index == numTabs) {
    var QuickNoteTab = gBrowser.addTab('chrome://quicknote/content/quicknote.xhtml');
    QuickNoteTab.setAttribute('QuickNote', 'true');
    getBrowser().selectedTab = QuickNoteTab;
  }
}


function QuickNote_closeWindow()
{
  window.close();
}


// toUnicode()
//  convert given string (s) from QuickNote_saveCharset to Unicode, which
//  will then be used in the QN editboxes
//Contributed by Leung WC
function toUnicode(s) {
  const UConvID = '@mozilla.org/intl/scriptableunicodeconverter';
  const UConvIF  = Components.interfaces.nsIScriptableUnicodeConverter;
  const UConv = Components.classes[UConvID].getService(UConvIF);
  UConv.charset = QN_Preferences.QuickNote_saveCharset;
  var out = UConv.ConvertToUnicode(s);
  return out;
}
//
//Contributed by Leung WC
function fromUnicode(s) {
  const UConvID = '@mozilla.org/intl/scriptableunicodeconverter';
  const UConvIF  = Components.interfaces.nsIScriptableUnicodeConverter;
  const UConv = Components.classes[UConvID].getService(UConvIF);
  UConv.charset = QN_Preferences.QuickNote_saveCharset;
  var out = UConv.ConvertFromUnicode(s);
  return out;
}

function QuickNote_getTextareaText(tNodeID) {
  var tat = fromUnicode(QN_Document.getElementById(tNodeID).value);
  return tat;
}

function QuickNote_setTextareaText(thetext,tNodeID) {
  QN_Document.getElementById(tNodeID).value = toUnicode(thetext);
}


// applies visual customizations for a tab (bgcolor, fonts, tab names etc.)
function QuickNote_applyCustomizations(Note) {
  var NodeID = "Pad" + Note;
  var thePad = QN_Document.getElementById(NodeID);
  var oldStyles = thePad.getAttribute("style");
  thePad.setAttribute("style",
    oldStyles + " " +
    "font: " + QN_Preferences.QN_padFontSize[Note].toString() + "px " +
    QN_Preferences.QN_padFontName[Note] + "; "+
    "color:" + QN_Preferences.QN_padFontColor[Note] + "; " +
    "-moz-appearance: none; background-color: "+ QN_Preferences.QN_padBackColor[Note] + "; " +
    QN_Preferences.QN_padUserStyle[Note]);
  QN_Document.getElementById("Tab"+Note).label = QN_Preferences.QN_tabName[Note];
}

////////////////////////////////////////////////////////////////////////////////
// loadNote()
//
// Desc: This function decides whether to load user defined files or create 
//       new quicknote[1-4].txt in the profile chrome directory.
// Params: Note - which note to load (1..4)
////////////////////////////////////////////////////////////////////////////////
function QuickNote_loadNote(Note) {
  var userTabpath = QN_Preferences.QN_tabFile[Note];
  if( !userTabpath || userTabpath=='Blank' ) { // todo asq: get rid of 'Blank'
    QuickNote_setDefaultFile(Note);
    /*
    var s=
    "\nqnfirststart="+QN_Preferences.qnfirststart +
    "\ntabtosendto="+QN_Preferences.tabtosendto +
    "\nnumberoftabs="+QN_Preferences.numberoftabs +
    "\nhowtopenQN="+QN_Preferences.howtopenQN;

    // ? getLocalizedUnicharPref vs copyUnicharPref
    for(var i=1; i<=QN_notesCount; i++) {
      s=s+"\nQN_tabFile="+QN_Preferences.QN_tabFile[i];
      s=s+"\nQN_padFontSize[="+QN_Preferences.QN_padFontSize[i];
      s=s+"\nQN_padFontName="+QN_Preferences.QN_padFontName[i];
      s=s+"\nQN_padFontColor="+QN_Preferences.QN_padFontColor[i];
      s=s+"\nQN_padUserStyle="+QN_Preferences.QN_padUserStyle[i];
      s=s+"\nQN_padBackColor="+QN_Preferences.QN_padBackColor[i];
      s=s+"\nQN_tabName="+QN_Preferences.QN_tabName[i];
    }
    s=s+"\nQuickNote_saveCharset="+QN_Preferences.QuickNote_saveCharset;
    s=s+"\nQNLastTabOpened="+QN_Preferences.QNLastTabOpened;
    s=s+"\nQN_hideButtonBox="+QN_Preferences.QN_hideButtonBox;
    s=s+"\nQN_autosave="+QN_Preferences.QN_autosave;
    QuickNote_setTextareaText(s, 'Pad'+Note);
    */
  } else {
    QuickNote_loadAFile(userTabpath, Note);
  }
  QuickNote_applyCustomizations(Note);
}

////////////////////////////////////////////////////////////////////////////////
// setDefaultFile()
//
// Desc: Used for loading the default Profile Notes, sets the dir and calls 
//       loadFromFile() to actually load the files.
// Parameters: 
//    Note: Where it is headed to (1..4)
////////////////////////////////////////////////////////////////////////////////
function QuickNote_setDefaultFile(Note) {//!! review/cleanup
  var fileName = "quicknote" + Note + ".txt";
  var d = new DirUtils;
  var UserChrome = d.getUserChromeDir();
  var f = new File(UserChrome);
  var target = f.nsIFile;
  target.append(fileName);
  f = new File(target.path);

  var targetNodeID = "Pad" + Note;
  if(f.exists())
  {  // if file already exists then just open it.
    QuickNote_loadFromFile(f,Note);
    QN_Preferences.QN_tabFile[Note] = f.path;
    nsPreferences.setUnicharPref('quicknote.tab'+Note+'path', f.path);
  } else if(!f.exists())
  { // If it doesn't exist, create the file, then make sure it exists, then open.
    f.create();
    if (f.exists()) {
      QuickNote_loadFromFile(f,Note);
      QN_Preferences.QN_tabFile[Note] = f.path;
      nsPreferences.setUnicharPref('quicknote.tab'+Note+'path', f.path);
    } else {
      var textAlert =
        QN_stringBundle.getFormattedString("qnerr.cantCreate", [f.path])+'\n'+
        QN_stringBundle.getString("qnerr.cantCreate.hint");
      QuickNote_setTextareaText(textAlert, 'Pad'+targetNodeID);
    }
  }
}


////////////////////////////////////////////////////////////////////////////////
// loadAFile
//
// Desc: Loads userdefined files (set in prefs)
// Parameters: 
//    userFileName: The name and location of the file to load.
//    Note: Where it is headed to (1..4)
// Returns: true or false
////////////////////////////////////////////////////////////////////////////////
function QuickNote_loadAFile(userFileName, Note) {
  if(!userFileName || userFileName=='Blank') {
    //!! these statements never get executed
    var textAlert = QN_stringBundle.getString("qnerr.noFile");
    QuickNote_setTextareaText(textAlert, 'Pad'+Note);
    return false;
  }

  var f = new File(userFileName);
  var target = f.nsIFile;
  f = new File(target.path);

  if (f.exists()) {  //If file already exists then just open it.
    QuickNote_loadFromFile(f, Note);
  } else { //If it doesn't exist, tell the user
           // todo: maybe try to create the file instead?
    var textAlert =
      QN_stringBundle.getFormattedString("qnerr.invalidFile", [f.path])+'\n'+
      QN_stringBundle.getString("qnerr.invalidFile.hint");
    QuickNote_setTextareaText(textAlert, 'Pad'+Note);
  }
  return true;
}

////////////////////////////////////////////////////////////////////////////////
// loadFromFile
// Desc: Actually loads the file thanks to JSLib
// Parameters:
//    f: File (JSLib)
//    Note: where the file should be loaded to (1..4)
// Returns: true if file saved, false otherwise
////////////////////////////////////////////////////////////////////////////////
// load from f (of type File) f to the node specified with targetNodeID.
function QuickNote_loadFromFile(f,Note) {
  try {
    QN_charsLeft[Note] = QN_Preferences.QN_autosave;
    f.open();
    var stuff = f.read();
    f.close();
    QuickNote_setTextareaText(stuff,"Pad"+Note);
    QN_noteLength[Note] = QN_Document.getElementById("Pad"+Note).textLength;
  } catch(e) {
    var textAlert =
      QN_stringBundle.getFormattedString("qnerr.failedLoadFile", [f.path])+'\n'+
      QN_stringBundle.getFormattedString("qnerr.failedLoadFile.hint", [QN_Preferences.QuickNote_saveCharset]);
    QuickNote_setTextareaText(textAlert,'Pad'+Note);
  }
}



////////////////////////////////////////////////////////////////////////////////
// renameCurrentTab
//
// Desc: Rename the current Tab
// Sets prefs quicknote.tab1name, .tab2name, etc.
////////////////////////////////////////////////////////////////////////////////
function QuickNote_renameCurrentTab() {
  var Note = QN_Document.getElementById("qntabs").selectedIndex+1;//current note number
  var currentTabName = QN_Document.getElementById("tabbox").selectedTab.label;
  var newTabName;

  var promptText = QN_stringBundle.getFormattedString("qn.tabRenamePrompt", [currentTabName]);
  newTabName = prompt(promptText, currentTabName);
  if(newTabName != null) { // Set up new name
    QN_Document.getElementById("tabbox").selectedTab.label = newTabName;
    var prefName = "quicknote.tab" + Note + "name";
    nsPreferences.setUnicharPref(prefName, newTabName);
  }
}

// returns the index (1..4) of current note
function QN_currentNote()
{
  // todo: use this whenever possible
  return QN_Document.getElementById("qntabs").selectedIndex+1;//selIndex is 0-based!
}

////////////////////////////////////////////////////////////////////////////////
// saveCurrent
//
// Desc: Saves the active pad to a corresponding file through a call to saveNote
//       Used for Ctrl+S ('Save Current Note') handling.
////////////////////////////////////////////////////////////////////////////////
function QuickNote_saveCurrent()
{
  QuickNote_saveNote(QN_Document.getElementById("qntabs").selectedIndex+1);
}


////////////////////////////////////////////////////////////////////////////////
// saveAll
// Desc: Saves all tabs using SaveNote
////////////////////////////////////////////////////////////////////////////////
function QuickNote_saveAll()
{
  for(var i=1; i<=QN_notesCount; i++)
    QuickNote_saveNote(i);
}

////////////////////////////////////////////////////////////////////////////////
// saveNote
//
// Desc: Saves a note. Utilized the saveAFile function for actual saving
// Parameters: Note - the number of the note to be save (1..4)
// Returns: nothing
////////////////////////////////////////////////////////////////////////////////
function QuickNote_saveNote(Note) {
  var fName = QN_Preferences.QN_tabFile[Note];
  var p = QuickNote_saveAFile(fName, Note);
  if(p) {
    QN_charsLeft[Note] = QN_Preferences.QN_autosave;
    QN_Document.getElementById('save'+Note).setAttribute('disabled', 'true');
  }
}

////////////////////////////////////////////////////////////////////////////////
// autoSave()
//
// Desc: Auto-save on close function.
//       Saves all the [modified] files via saveNote().
//       Also saves the active tab information.
////////////////////////////////////////////////////////////////////////////////
function QuickNote_autoSave() {
  QN_Preferences.QNLastTabOpened = QN_Document.getElementById("qntabs").
    selectedIndex+1; //note that selIndex is 0-based.
    
  nsPreferences.setIntPref('quicknote.lasttabopen', QN_Preferences.QNLastTabOpened);

  if(QN_Preferences.QN_autosave < 0)
    return;

  for(var i=1; i<=QN_notesCount; i++) {
    if (!QN_Document.getElementById('save'+i).hasAttribute('disabled'))
      QuickNote_saveNote(i);
  }
}

////////////////////////////////////////////////////////////////////////////////
// onClose()
//  used when 'no autosave' option is selected
//  shows a dialog to let user select the notes he wishes to save
////////////////////////////////////////////////////////////////////////////////
function QuickNote_onClose()
{
  // since this file (and this function in particular) is used in global
  // overlay (overlay.xul), we need check if the prefs have changed.
  // xxx the same issue may show up elsewhere
  QN_Preferences.QN_autosave = nsPreferences.getIntPref("quicknote.autosave");

  if(QN_Preferences.QN_autosave < 0) {
    // we get ok/cancel return value from the dialog through re.val
    var re = {};
    var Show = Array(5);  //whether the i-th note was available in QuickNote
    var Save = Array(5);  //was it modified?
    var Changed = false;  //anything modified?
    var i;
    
    for(i=1;i<=QN_Preferences.numberoftabs;i++)       Show[i] = true;
    for(i=QN_Preferences.numberoftabs+1;i<=4;i++)     Show[i] = false;
    for(i=1; i<=4; i++)
    {
      Save[i] = !QN_Document.getElementById('save'+i).hasAttribute('disabled');
      Changed = Changed || Save[i];
    }
    if(!Changed) return true; // everything is already saved => close right away
    
    window.openDialog("chrome://quicknote/content/qnconfirmsave.xul",
      "qnconfirmsave","chrome,modal,centerscreen",re,Show,QN_Preferences.QN_tabName,Save);
    if(!re.val)       // cancel: don't close QuickNote
      return false;
    else {            // ok: save the selected notes
      for(i=1; i<=4; i++)
        if(Save[i])
          QuickNote_saveNote(i);
    }
  }
  return true;
}

// onInput()
// Desc: "autosave every X characters" function.
// Note: deltaLen et al are used to handle pasting large text blocks
// from clipboard correctly and reliably :)
function QN_onInput(Note) {
  var thePad = QN_Document.getElementById("Pad" + Note);
  if(QN_Preferences.QN_autosave > 0) // if autosave as you type is enabled...
  {
    var deltaLen = Math.abs(QN_noteLength[Note]-thePad.textLength);
    QN_charsLeft[Note] = QN_charsLeft[Note] - Math.max(deltaLen,1);
    if(QN_charsLeft[Note] <= 0) { // it's time to auto-save the note
      QuickNote_saveNote(Note);
    } else {
      QN_Document.getElementById('save' + Note).removeAttribute('disabled');
    }
  } else
    QN_Document.getElementById('save' + Note).removeAttribute('disabled');
  QN_noteLength[Note] = thePad.textLength;
}


////////////////////////////////////////////////////////////////////////////////
// saveNoteAs
// Desc: Saves a Note to a user-defined location via saveAFile().
// Utilized saveAFile for actual saving.
//
// Parameters: Note: What to save (number, 1..4)
// Returns: true if list saved, false otherwise
//
// Originally Modeled after the SaveAs code from Tagzilla.mozdev.org!
////////////////////////////////////////////////////////////////////////////////
function QuickNote_saveNoteAs(Note) {
  var d = new DirUtils;
  var myUserDirPlease = d.getUserChromeDir();
  // todo: Note -> note title
  var promptText = QN_stringBundle.getFormattedString("qn.saveAsPrompt", [Note]);
  // xxx Hey, txtFilePicker() wants only two parameters!
  var fName = QuickNote_txtFilePicker(promptText, 1, myUserDirPlease);
  if(fName == null)
    return false;

  var isSaved = QuickNote_saveAFile(fName, Note);
  return isSaved;
}


////////////////////////////////////////////////////////////////////////////////
// saveAFile
// Desc: Actually does the filesaving thanks to JSLib
// Parameters:
//    aUrl: string name of file to save, in platform-specific or file:// format
//    Note: where the data is to save (1..4)
// Returns: true if file saved, false otherwise
////////////////////////////////////////////////////////////////////////////////
function QuickNote_saveAFile(aUrl, Note) {
  var aFile = aUrl;
  if(aUrl.substring(0,7) == "file://") {
    var fUtils = new FileUtils();
    aFile = fUtils.urlToPath(aUrl);
  }

  var f = new File(aFile);
  if(!f.open("w")) {
    var alertText = QN_stringBundle.getFormattedString("qnerr.cantOpenw", [aFile]);
    alert(alertText);
    // todo: perhaps throw an exception to prevent closing QN and losing data that couldn't be saved?
    return false;
  }

  var data = QuickNote_getTextareaText("Pad" + Note);
  var p;
  // File::write() returns false if data==null
  // Still it is ok to save a blank file
  // (Note:  File::open('w') overwrites the file if it already exists)
  if(data) {
    p = f.write(data);
  } else {
    p = true;
  }
  f.close();
  return p;
}

////////////////////////////////////////////////////////////////////////////////
// catchText
//
// Desc: Catch text sent via context menu. Append it to the note specified with
//       tabtosendto.
// Parameters: QuickNote window; text to catch
// Returns: the pab-id which got the text; null when no text to catch.
// ?? make it return true/false ?? the return value is only tested against null...
///////////////////////////////////////////////////////////////////////////////
function QuickNote_catchText(text) {
  var target; // what node to send text to, 1..4

  if(QN_Preferences.tabtosendto)
    target = QN_Preferences.tabtosendto;
  else
    target = '1';

  if(text) {  // This is causing the extra /n to apear; todo
    QN_Preferences.QNLastTabOpened = nsPreferences.setIntPref('quicknote.lasttabopen', target);
    var Pad = QN_Document.getElementById("Pad" + target);
    Pad.value = Pad.value + "\n" + text + "\n";
    QuickNote_saveNote(target);//todo: am I sure? (when autosave is off, user might not want the note to be saved)

    // current behaviour is to pop the quicknote window on 'send text'
    // so we will also switch to the tab, that 'caught' the text:
    QN_Document.getElementById("qntabs").selectedIndex = target-1;
    Pad.focus();

    return "Pad" + target;
  }
  return null;
}



////////////////////////////////////////////////////////////////////////////////
// txtFilePicker
//
// Parameters:
//  aTitle: title to go on file picker window
//  aSave: 1 if picking file to save/overwrite, 0 if picking file to load
// No longer used. --> aStart: directory to start from No longer used.
// (kept as a resource)    This must be an nsILocalFile.  new Dir("foo") in jslib/io/dir.js will
//          do the trick, but you have to do it yourself.
// Returns:
//  Name of file picked, in URL format, or null if cancelled
// Original taken and modified from Tagzilla.mozdev.org!
////////////////////////////////////////////////////////////////////////////////
function QuickNote_txtFilePicker(aTitle, aSave) {
  var retVal = null;
  try {
    const nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
    fp.init(window, aTitle, (aSave ? nsIFilePicker.modeSave : nsIFilePicker.modeOpen));
    fp.appendFilters(nsIFilePicker.filterAll | nsIFilePicker.filterText);

    /*
    if(aStart) {
      fp.displayDirectory = aStart;
    }
    */
    var result=fp.show();
    if (result == nsIFilePicker.returnOK || result == nsIFilePicker.returnReplace) {
      //retVal=fp.fileURL.spec;
      retVal=fp.file.path;
    }
  }
  catch (e) {
  }
  return retVal;
}


/*
////////////////////////////////////////////////////////////////////////////////
// mailIt
//  **********NOT IN USE YEST*******
// Parameters:
//  targetNodeID: where the data is to save!
// Returns: true
// To-Do: TESTING, can't get to get Thunderbird to load a Compose window
// Haven't tested since TB 0.1. Will do after 0.6 is released
////////////////////////////////////////////////////////////////////////////////
function QuickNote_qnMailWindow(targetNodeID) {
  var QNmailText = QuickNote_getTextareaText(targetNodeID).value;
  window.openDialog("chrome://quicknote/content/mail.xul",
      "_blank", "chrome,all,dialog", QNmailText);
  return true;
}
*/

